@@ -1,6 +1,5 @@
-#ifdef USE_INSULTS
 /*
- *  CU sudo version 1.3
+ *  CU sudo version 1.3.1
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,6 +18,11 @@
  *  Please send bugs, changes, problems to sudo-bugs.cs.colorado.edu
  */
 
+#ifdef USE_INSULTS
+
+#ifndef _SUDO_INSULTS_H
+#define _SUDO_INSULTS_H
+
 /*
  * To add insult to injury, just add to the following strings and
  * adjust NOFINSULTS accordingly.  This code taken from the original
@@ -61,4 +65,6 @@ char *insults[] = {
 
 #define INSULT		(insults[time(NULL) % NOFINSULTS])
 
+#endif /* _SUDO_INSULTS_H */
+
 #endif /* USE_INSULTS */
