@@ -1,5 +1,5 @@
 /*
- * CU sudo version 1.3 (based on Root Group sudo version 1.1)
+ * CU sudo version 1.3.1 (based on Root Group sudo version 1.1)
  *
  * This software comes with no waranty whatsoever, use at your own risk.
  *
@@ -37,27 +37,44 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: logging.c,v 1.7 1994/02/09 03:02:57 millert Exp $";
+static char rcsid[] = "$Id: logging.c,v 1.28 1994/08/12 01:58:58 millert Exp $";
 #endif /* lint */
 
+#include "config.h"
+
 #include <stdio.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif /* HAVE_UNISTD_H */
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif /* HAVE_STRING_H */
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif /* HAVE_STRINGS_H */
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/errno.h>
+#include <netinet/in.h>
+
 #include "sudo.h"
 
-void log_error();
-void readchild();
-static void send_mail();
-static void reapchild();
-static int appropriate();
+/*
+ * Prototypes for local functions
+ */
+static void send_mail		__P((void));
+static RETSIGTYPE reapchild	__P((int));
+static int appropriate		__P((int));
 
+/*
+ * Globals
+ */
 static char logline[MAXLOGLEN + 8];
 
+
 /**********************************************************************
  *
  *  log_error()
@@ -69,70 +86,62 @@ static char logline[MAXLOGLEN + 8];
 void log_error(code)
     int code;
 {
-    char cwd[MAXPATHLEN + 1];
     int argc;
     char **argv;
+    mode_t oldmask;
     register char *p;
     register int count;
-#ifndef SYSLOG
+#ifdef _PATH_SUDO_LOGFILE
     register FILE *fp;
     time_t now;
-#else
+#endif /* _PATH_SUDO_LOGFILE */
+#ifdef SYSLOG
     register int pri;		/* syslog priority */
-#endif
+    char *tmp, save;
+#endif /* SYSLOG */
 
     /*
      * there is no need to log the date and time twice if using syslog
      */
-#ifndef SYSLOG
+#ifdef _PATH_SUDO_LOGFILE
     now = time((time_t) 0);
     (void) sprintf(logline, "%19.19s : %8.8s : ", ctime(&now), user);
-#else
-    (void) sprintf(logline, "%8.8s : ", user);
-#endif
+#endif /* _PATH_SUDO_LOGFILE */
 
     /*
      * we need a pointer to the end of logline
      */
     p = logline + strlen(logline);
 
-    /*
-     * so we know where we are...
-     */
-#ifdef USE_CWD
-    getcwd(cwd, (size_t) (MAXPATHLEN + 1));
-#else
-    getwd(cwd);
-#endif
-
     switch (code) {
 
 	case ALL_SYSTEMS_GO:
 	    (void) sprintf(p, "PWD=%s ; COMMAND=", cwd);
 #ifdef SYSLOG
 	    pri = Syslog_priority_OK;
-#endif
+#endif /* SYSLOG */
 	    break;
 
 	case VALIDATE_NO_USER:
 	    (void) sprintf(p, "user NOT in sudoers ; PWD=%s ; COMMAND=", cwd);
 #ifdef SYSLOG
 	    pri = Syslog_priority_NO;
-#endif
+#endif /* SYSLOG */
 	    break;
 
 	case VALIDATE_NOT_OK:
 	    (void) sprintf(p, "command not allowed ; PWD=%s ; COMMAND=", cwd);
 #ifdef SYSLOG
 	    pri = Syslog_priority_NO;
-#endif
+#endif /* SYSLOG */
 	    break;
 
 	case VALIDATE_ERROR:
-	    (void) sprintf(p, "error in %s ; PWD=%s ; command: ", SUDOERS, cwd);
+	    (void) sprintf(p, "error in %s ; PWD=%s ; command: ",
+		_PATH_SUDO_SUDOERS, cwd);
 #ifdef SYSLOG
 	    pri = Syslog_priority_NO;
-#endif
+#endif /* SYSLOG */
 	    break;
 
 	case GLOBAL_NO_PW_ENT:
@@ -140,7 +149,7 @@ void log_error(code)
 		uid);
 #ifdef SYSLOG
 	    pri = Syslog_priority_NO;
-#endif
+#endif /* SYSLOG */
 	    break;
 
 	case PASSWORD_NOT_CORRECT:
@@ -148,20 +157,21 @@ void log_error(code)
 		    TRIES_FOR_PASSWORD, cwd);
 #ifdef SYSLOG
 	    pri = Syslog_priority_NO;
-#endif
+#endif /* SYSLOG */
 	    break;
 
 	case GLOBAL_NO_HOSTNAME:
 	    strcat(p, "This machine does not have a hostname ");
 #ifdef SYSLOG
 	    pri = Syslog_priority_NO;
-#endif
+#endif /* SYSLOG */
 	    break;
 
 	case NO_SUDOERS_FILE:
 	    switch (errno) {
 		case ENOENT:
-		    (void) sprintf(p, "There is no %s file.  ", SUDOERS);
+		    (void) sprintf(p, "There is no %s file.  ",
+			_PATH_SUDO_SUDOERS);
 		    break;
 		case EACCES:
 		    (void) sprintf(p, "%s needs to run setuid root.  ",
@@ -169,26 +179,26 @@ void log_error(code)
 		    break;
 		default:
 		    (void) sprintf(p, "There is a problem opening %s ",
-			SUDOERS);
+			_PATH_SUDO_SUDOERS);
 		    break;
 	    }
 #ifdef SYSLOG
 	    pri = Syslog_priority_NO;
-#endif
+#endif /* SYSLOG */
 	    break;
 
 	case GLOBAL_HOST_UNREGISTERED:
 	    (void) sprintf(p, "gethostbyname() cannot find host %s ", host);
 #ifdef SYSLOG
 	    pri = Syslog_priority_NO;
-#endif
+#endif /* SYSLOG */
 	    break;
 
 	default:
 	    strcat(p, "found a wierd error : ");
 #ifdef SYSLOG
 	    pri = Syslog_priority_NO;
-#endif
+#endif /* SYSLOG */
 	    break;
     }
 
@@ -208,10 +218,11 @@ void log_error(code)
 	count = (int) (logline + MAXLOGLEN - p);
 
 	/*
-	 * now stuff as much of the rest of the line as will fit
+	 * Now stuff as much of the rest of the line as will fit
+	 * Do word wrap if logging to a file.
 	 */
 	while (count > 0 && argc--) {
-	    strncpy(p, *++argv, count);
+	    strncpy(p, *(++argv), count);
 	    strcat(p, " ");
 	    p += 1 + (count < strlen(*argv) ? count : strlen(*argv));
 	    count = (int) (logline + MAXLOGLEN - p);
@@ -220,32 +231,130 @@ void log_error(code)
 	    strcat(p, " ... ");	/* add an elipsis to the end */
 
     }
-    if (appropriate(code))
-	send_mail();
 
 #ifdef SYSLOG
+#ifdef Syslog_facility
     openlog(Syslog_ident, Syslog_options, Syslog_facility);
-    syslog(pri, logline);
-    closelog();
 #else
+    openlog(Syslog_ident, Syslog_options);
+#endif /* Syslog_facility */
+
+    /*
+     * Log the full line, breaking into multiple syslog(3) calls if necesary
+     */
+    p = (char *)logline;
+    for (count=0; count < (strlen(logline) / MAXSYSLOGLEN) + 1; count++) {
+	if (strlen(p) > MAXSYSLOGLEN) {
+	    /*
+	     * Break up the line into what will fit on one syslog(3) line
+	     * Try to break on a word boundary if possible.
+	     */
+	    for (tmp = p + MAXSYSLOGLEN; tmp > p && *tmp != ' '; tmp--)
+		;
+	    if (tmp <= p)
+		tmp = p + MAXSYSLOGLEN;
+
+	    /* NULL terminate line, but save the char to restore later */
+	    save = *tmp;
+	    *tmp = '\0';
+
+	    if (count == 0)
+		syslog(pri, "%8.8s : %s", user, p);
+	    else
+		syslog(pri, "%8.8s : (command continued) %s", user, p);
+
+	    *tmp = save;			/* restore saved character */
+
+	    /* eliminate leading whitespace */
+	    for (p=tmp; *p != ' '; p++)
+		;
+	} else {
+	    if (count == 0)
+		syslog(pri, "%8.8s : %s", user, p);
+	    else
+		syslog(pri, "%8.8s : (command continued) %s", user, p);
+	}
+    }
+    closelog();
+#endif /* SYSLOG */
+#ifdef _PATH_SUDO_LOGFILE
+
     /* become root */
     be_root();
 
-    if ((fp = fopen(LOGFILE, "a")) == NULL) {
-	(void) sprintf(logline, "Can\'t open log file: %s", LOGFILE);
+    oldmask = umask(077);
+    fp = fopen(_PATH_SUDO_LOGFILE, "a");
+    (void) umask(oldmask);
+    if (fp == NULL) {
+	(void) sprintf(logline, "Can\'t open log file: %s", _PATH_SUDO_LOGFILE);
 	send_mail();
     } else {
-	(void) fprintf(fp, "%s\n", logline);
+	char *beg, *oldend, *end;
+	register int maxlen = MAXLOGFILELEN;
+
+	/*
+	 * Print out logline with word wrap
+	 */
+	beg = end = logline;
+	while (beg) {
+	    oldend = end;
+	    end = strchr(oldend, ' ');
+
+	    if (end) {
+		*end = '\0';
+		if (strlen(beg) > maxlen) {
+		    /* too far, need to back up & print the line */
+
+		    if (beg == (char *)logline)
+			maxlen -= 4;		/* don't indent first line */
+
+		    *end = ' ';
+		    if (oldend != beg) {
+			/* rewind & print */
+		    	end = oldend-1;
+			while (*end == ' ')
+			    --end;
+			*(++end) = '\0';
+			(void) fprintf(fp, "%s\n    ", beg);
+			*end = ' ';
+		    } else {
+			(void) fprintf(fp, "%s\n    ", beg);
+		    }
+
+		    /* reset beg to point to the start of the new substring */
+		    beg = end;
+		    while (*beg == ' ')
+			++beg;
+		} else {
+		    /* we still have room */
+		    *end = ' ';
+		}
+
+		/* remove leading whitespace */
+		while (*end == ' ')
+		    ++end;
+	    } else {
+		/* final line */
+		(void) fprintf(fp, "%s\n", beg);
+		beg = NULL;			/* exit condition */
+	    }
+	}
+
 	(void) fclose(fp);
     }
 
     /* relinquish root */
     be_user();
-#endif
+#endif /* _PATH_SUDO_LOGFILE */
+
+    /* send mail if appropriate */
+    if (appropriate(code))
+	send_mail();
 }
 
 
 
+#ifdef MAILER
 /**********************************************************************
  *
  *  send_mail()
@@ -254,10 +363,10 @@ void log_error(code)
  *
  */
 
-char *exec_argv[] = {"sendmail",
-		     "-t",
-		     ALERTMAIL,
-		     (char *) NULL};
+static char *mail_argv[] = { "sendmail",
+			     "-t",
+			     ALERTMAIL,
+			     (char *) NULL };
 
 static void send_mail()
 {
@@ -265,20 +374,24 @@ static void send_mail()
     char *subject = MAILSUBJECT;
     int fd[2];
     char buf[MAXLOGLEN + 1024];
+#ifdef POSIX_SIGNALS
+    struct sigaction action;
 
-    /* become root for find_path() */
-    be_root();
+    (void) bzero((char *)(&action), sizeof(action));
+#endif /* POSIX_SIGNALS */
 
     if ((mailer = find_path(mailer)) == NULL) {
 	(void) fprintf(stderr, "%s: mailer (%s) not found\n", Argv[0], mailer);
 	exit(1);
     }
 
-    /* relinquish root */
-    be_user();
-
     /* catch children as they die */
+#ifdef POSIX_SIGNALS
+    action.sa_handler = reapchild;
+    (void) sigaction(SIGCHLD, &action, NULL);
+#else
     (void) signal(SIGCHLD, reapchild);
+#endif /* POSIX_SIGNALS */
 
     if (fork())
 	return;
@@ -288,9 +401,16 @@ static void send_mail()
      */
     be_full_user();
     
+#ifdef POSIX_SIGNALS
+    action.sa_handler = SIG_IGN;
+    (void) sigaction(SIGHUP, &action, NULL);
+    (void) sigaction(SIGINT, &action, NULL);
+    (void) sigaction(SIGQUIT, &action, NULL);
+#else
     (void) signal(SIGHUP, SIG_IGN);
     (void) signal(SIGINT, SIG_IGN);
     (void) signal(SIGQUIT, SIG_IGN);
+#endif /* POSIX_SIGNALS */
 
     if (pipe(fd)) {
 	perror("send_mail: pipe");
@@ -303,10 +423,10 @@ static void send_mail()
 
     if (!fork()) {		/* child */
 	(void) close(1);
-	execve(mailer, exec_argv, Envp);
+	EXEC(mailer, mail_argv);
 
 	/* this should not happen */
-	perror("execve");
+	perror(mailer);
 	exit(1);
     } else {			/* parent */
 	(void) close(0);
@@ -320,6 +440,13 @@ static void send_mail()
 	exit(0);
     }
 }
+#else
+static void send_mail()
+{
+    /* no mailer defined */
+    return;
+}
+#endif /* MAILER */
 
 
 
@@ -327,12 +454,16 @@ static void send_mail()
  *
  *  reapchild()
  *
- *  This function gets rid fo all the ugly zombies
+ *  This function gets rid of all the ugly zombies
  */
 
-static void reapchild()
+static RETSIGTYPE reapchild(sig)
+    int sig;
 {
-        (void) wait(NULL);
+    (void) wait(NULL);
+#ifndef POSIX_SIGNALS
+    (void) signal(SIGCHLD, reapchild);
+#endif /* POSIX_SIGNALS */
 }
 
 
@@ -372,6 +503,11 @@ void inform_user(code)
 		    "Intruder Alert!  You don\'t exist in the passwd file\n\n");
 	    break;
 
+	case GLOBAL_NO_AUTH_ENT:
+	    (void) fprintf(stderr,
+		    "Intruder Alert!  You don\'t exist in the auth database\n\n");
+	    break;
+
 	case GLOBAL_NO_HOSTNAME:
 	    (void) fprintf(stderr,
 		    "This machine does not have a hostname\n\n");
